@@ -1,7 +1,7 @@
 /* 
    Project: DataBasin
 
-   Copyright (C) 2008-2012 Free Software Foundation
+   Copyright (C) 2008-2013 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -29,10 +29,14 @@
 #import "DBSoapCSV.h"
 #import "DBCVSWriter.h"
 #import "DBCVSReader.h"
+#import "DBLogger.h"
+#import "DBProgress.h"
 
 #define DB_ENVIRONMENT_PRODUCTION 0
 #define DB_ENVIRONMENT_SANDBOX    1
 
+#define MAX_STORED_LOGINS 5
+
 #if defined(__APPLE__) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
 #define NSUTF16StringEncoding 999
 #endif
@@ -59,6 +63,27 @@
 {
   if ((self = [super init]))
     {
+      id logLevelObj;
+      NSUserDefaults *defaults;
+
+      defaults = [NSUserDefaults standardUserDefaults];
+
+      logger = [[DBLogger alloc] init];
+      logLevelObj = [defaults objectForKey: @"LogLevel"];
+      /* if the log level is not set we set it to the standard level */
+      if (logLevelObj == nil)
+        {
+          logLevelObj = [NSNumber numberWithInt: LogStandard];
+          [defaults setObject:logLevelObj forKey: @"LogLevel"];
+        }
+      [logger setLogLevel: [logLevelObj intValue]];
+
+      loginDict = [defaults objectForKey: @"logins"];
+      if (loginDict == nil)
+	loginDict = [NSMutableDictionary dictionary];
+      else
+	loginDict = [NSMutableDictionary dictionaryWithDictionary:loginDict];
+      [loginDict retain];
     }
   return self;
 }
@@ -67,6 +92,8 @@
 {
   [dbCsv release];
   [db release];
+  [logger release];
+  [loginDict release];
   [super dealloc];
 }
 
@@ -125,6 +152,24 @@
 
   [buttPrefHttps setState: [[defaults valueForKey: @"UseHttps"] intValue]];
 
+  index = 0;
+  switch([[defaults valueForKey: @"LogLevel"] intValue])
+    {
+      case LogStandard:
+        index = 0;
+        break;
+      case LogInformative:
+        index = 1;
+        break;
+      case LogDebug:
+        index = 2;
+        break;
+      default:
+        NSLog(@"Unexpected log level");
+        break;
+    }
+  [popupLogLevel selectItemAtIndex: index];
+
   [prefPanel makeKeyAndOrderFront: sender];
 }
 
@@ -136,6 +181,7 @@
 - (IBAction)prefPanelOk:(id)sender
 {
   NSStringEncoding selectedEncoding;
+  DBLogLevel selectedLogLevel;
   NSUserDefaults *defaults;
 
   defaults = [NSUserDefaults standardUserDefaults];
@@ -156,6 +202,22 @@
   [defaults setObject:[NSNumber numberWithInt: selectedEncoding] forKey: @"StringEncoding"];
 
   [defaults setObject:[NSNumber numberWithInt: [buttPrefHttps state]] forKey: @"UseHttps"];
+
+  selectedLogLevel = LogStandard;
+  switch([popupLogLevel indexOfSelectedItem])
+    {
+      case 0: selectedLogLevel = LogStandard;
+        break;
+      case 1: selectedLogLevel = LogInformative;
+        break;
+      case 2: selectedLogLevel = LogDebug;
+        break;
+      default:
+        break;
+    }
+  [defaults setObject:[NSNumber numberWithInt: selectedLogLevel] forKey: @"LogLevel"];
+  [logger setLogLevel: selectedLogLevel];
+
   [prefPanel performClose: nil];
 }
 
@@ -174,6 +236,11 @@
   [winUserInspector makeKeyAndOrderFront:self];
 }
 
+/* LOGGER */
+- (IBAction)showLog:(id)sender
+{
+  [logger show:sender];
+}
 
 /* LOGIN */
 
@@ -182,6 +249,29 @@
   [winLogin makeKeyAndOrderFront:self];
 }
 
+- (IBAction)usernameFieldAction:(id)sender
+{
+  NSDictionary *loginSet;
+  NSString *userName;
+  NSString *token;
+  NSNumber *envNum;
+
+  userName = [fieldUserName stringValue];
+  loginSet = [loginDict objectForKey:userName];
+  if (loginSet == nil)
+    return;
+  
+  [fieldPassword setStringValue:[loginSet objectForKey:@"password"]];
+  token = [loginSet objectForKey:@"token"];
+  if (token == nil)
+    token = @"";
+  [fieldToken setStringValue:token];
+
+  envNum = [loginSet objectForKey:@"environment"];
+  if (envNum)
+    [popupEnvironment selectItemAtIndex:[envNum intValue]];
+}
+
 - (IBAction)doLogin:(id)sender
 {
   NSString *userName;
@@ -192,13 +282,14 @@
   NSDictionary *uInfo;
   BOOL useHttps;
   NSString *protocolString;
+  NSMutableDictionary *loginSet;
   
   userName = [fieldUserName stringValue];
   password = [fieldPassword stringValue];
   token = [fieldToken stringValue];
 
   useHttps = NO;
-  if ([[[NSUserDefaults standardUserDefaults] valueForKey: @"UseHttps"] intValue] == NSOnState)
+  if ([[[NSUserDefaults standardUserDefaults] objectForKey: @"UseHttps"] intValue] == NSOnState)
     useHttps = YES;
 
   if (useHttps)
@@ -211,16 +302,17 @@
     password = [password stringByAppendingString:token];
     
   db = [[DBSoap alloc] init];
+  [db setLogger: logger];
   dbCsv = [[DBSoapCSV alloc] init];
   [dbCsv setDBSoap:db];
   
   urlStr = nil;
   if ([popupEnvironment indexOfSelectedItem] == DB_ENVIRONMENT_PRODUCTION)
-    urlStr = [protocolString stringByAppendingString: @"www.salesforce.com/services/Soap/u/20.0"];
+    urlStr = [protocolString stringByAppendingString: @"www.salesforce.com/services/Soap/u/25.0"];
   else if ([popupEnvironment indexOfSelectedItem] == DB_ENVIRONMENT_SANDBOX)
-    urlStr = [protocolString stringByAppendingString: @"test.salesforce.com/services/Soap/u/20.0"];
+    urlStr = [protocolString stringByAppendingString: @"test.salesforce.com/services/Soap/u/25.0"];
 
-  NSLog(@"Url: %@", urlStr);  
+  [logger log:LogStandard :@"[AppController doLogin] Url: %@\n", urlStr];  
   url = [NSURL URLWithString:urlStr];
   
   NS_DURING
@@ -246,12 +338,56 @@
     [fieldRoleId setStringValue: [uInfo valueForKey:@"roleId"]];
 
   NS_HANDLER
+    [logger log:LogStandard :@"Login failed\n"];
     if ([[localException name] hasPrefix:@"DB"])
       {
         [faultTextView setString:[localException reason]];
         [faultPanel makeKeyAndOrderFront:nil];
+	return;
+      }
+    else
+      {
+	NSLog(@"Unexpected exception: %@", [localException name]);
       }
   NS_ENDHANDLER
+  [logger log:LogStandard :@"[AppController doLogin] %@ logged in succesfully\n", userName];
+  
+  loginSet = [NSMutableDictionary dictionaryWithCapacity:4];
+  [loginSet retain];
+  [loginSet setObject:userName forKey:@"username"];
+  [loginSet setObject:[fieldPassword stringValue] forKey:@"password"];
+  if (token != nil)
+    [loginSet setObject:token forKey:@"token"];
+  [loginSet setObject:[NSDate date] forKey:@"lastlogin"];
+  [loginSet setObject:[NSNumber numberWithInt:[popupEnvironment indexOfSelectedItem]] forKey:@"environment"];
+  [loginDict setObject:loginSet forKey:userName];
+  [loginSet release];
+  NSLog(@"login dictionary is: %@", loginDict);
+  if ([loginDict count] > MAX_STORED_LOGINS)
+    { 
+      NSEnumerator *e;
+      id key;
+      id oldKey;
+
+      [logger log:LogInformative :@"[AppController doLogin] Maximum number of stored logins reached, removing oldest\n"];
+      e = [loginDict keyEnumerator];
+      oldKey = nil;
+      while ((key = [e nextObject]))
+	{
+	  NSDictionary *currSet;
+	  NSDictionary *oldSet;
+
+	  if (oldKey == nil)
+	    oldKey = key;
+	  currSet = [loginDict objectForKey:key];
+	  oldSet = [loginDict objectForKey:oldKey];
+	  if ([[currSet objectForKey:@"lastlogin"] compare: [oldSet objectForKey:@"lastlogin"]] == NSOrderedAscending)
+	    oldKey = key;
+	}
+      [logger log:LogInformative :@"[AppController doLogin] delete: %@\n", oldKey];
+      [loginDict removeObjectForKey:oldKey];
+    }
+  [[NSUserDefaults standardUserDefaults] setObject:loginDict forKey: @"logins"];
 }
 
 /* UPDATE SOBJECT LIST */
@@ -298,11 +434,10 @@
   NSFileHandle  *fileHandle;
   NSFileManager *fileManager;
   DBCVSWriter   *cvsWriter;
+  DBProgress    *progress;
   
   statement = [fieldQuerySelect string];
-  NSLog(@"%@", statement);
   filePath = [fieldFileSelect stringValue];
-  NSLog(@"%@", filePath);
   
   fileManager = [NSFileManager defaultManager];
   if ([fileManager createFileAtPath:filePath contents:nil attributes:nil] == NO)
@@ -317,9 +452,14 @@
       NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
     }
   
+  progress = [[DBProgress alloc] init];
+  [progress setLogger:logger];
+  [progress reset];
   cvsWriter = [[DBCVSWriter alloc] initWithHandle:fileHandle];
+  [cvsWriter setLogger:logger];
+
   NS_DURING
-    [dbCsv query :statement queryAll:([queryAllSelect state] == NSOnState) toWriter:cvsWriter];
+    [dbCsv query :statement queryAll:([queryAllSelect state] == NSOnState) toWriter:cvsWriter progressMonitor:progress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -329,6 +469,7 @@
   NS_ENDHANDLER
   [cvsWriter release];
   [fileHandle closeFile];
+  [progress release];
 }
 
 /* INSERT */
@@ -374,17 +515,22 @@
   NSString      *filePath;
   DBCVSReader   *reader;
   NSString      *intoWhichObject;
+  DBProgress    *progress;
   
   filePath = [fieldFileInsert stringValue];
   NSLog(@"%@", filePath);
   
   intoWhichObject = [[[popupObjectsInsert selectedItem] title] retain];
-  NSLog(@"object: %@", intoWhichObject);
+  [logger log:LogInformative :@"[AppController executeInsert] object: %@\n", intoWhichObject];
+
+  progress = [[DBProgress alloc] init];
+  [progress setLogger:logger];
+  [progress reset];
   
-  reader = [[DBCVSReader alloc] initWithPath:filePath];
+  reader = [[DBCVSReader alloc] initWithPath:filePath withLogger:logger];
   
   NS_DURING
-    [dbCsv create:intoWhichObject fromReader:reader];
+    [dbCsv create:intoWhichObject fromReader:reader progressMonitor:progress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -395,6 +541,7 @@
   
   [reader release];
   [intoWhichObject release];
+  [progress release];
 }
 
 /* UPDATE */
@@ -440,17 +587,22 @@
   NSString      *filePath;
   DBCVSReader   *reader;
   NSString      *whichObject;
+  DBProgress    *progress;
   
   filePath = [fieldFileUpdate stringValue];
   NSLog(@"%@", filePath);
   
+  progress = [[DBProgress alloc] init];
+  [progress setLogger:logger];
+  [progress reset];
+
   whichObject = [[[popupObjectsUpdate selectedItem] title] retain];
-  NSLog(@"object: %@", whichObject);
+  [logger log:LogInformative :@"[AppController executeUpdate] object: %@\n", whichObject];
   
-  reader = [[DBCVSReader alloc] initWithPath:filePath];
+  reader = [[DBCVSReader alloc] initWithPath:filePath withLogger:logger];
   
   NS_DURING
-    [dbCsv update:whichObject fromReader:reader];
+    [dbCsv update:whichObject fromReader:reader progressMonitor:progress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -461,6 +613,7 @@
     
   [reader release];
   [whichObject release];
+  [progress release];
 }
 
 /*  SELECT IDENTIFY */
@@ -508,14 +661,12 @@
   NSFileManager *fileManager;
   DBCVSWriter   *cvsWriter;
   DBCVSReader   *cvsReader;
-  int batchSize;
+  DBProgress    *progress;
+  int           batchSize;
   
   statement = [fieldQuerySelectIdentify string];
-  NSLog(@"%@", statement);
   filePathIn = [fieldFileSelectIdentifyIn stringValue];
-  NSLog(@"fpi: %@", filePathIn);
   filePathOut = [fieldFileSelectIdentifyOut stringValue];
-  NSLog(@"fpo: %@", filePathOut);
 
   batchSize = 0;
   switch ([[popupBatchSizeIdentify selectedItem] tag])
@@ -533,13 +684,13 @@
       batchSize = -1;
       break;
     default:
-      NSLog(@"unexpected batch size");
+      [logger log:LogStandard :@"[AppController executeSelectIdentify] unexpected batch size\n"];
     }
-  NSLog(@"batch Size: %d", batchSize);
+  [logger log:LogDebug :@"[AppController executeSelectIdentify] batch Size: %d\n", batchSize];
   
   fileManager = [NSFileManager defaultManager];
 
-  cvsReader = [[DBCVSReader alloc] initWithPath:filePathIn];
+  cvsReader = [[DBCVSReader alloc] initWithPath:filePathIn withLogger:logger];
 
   if ([fileManager createFileAtPath:filePathOut contents:nil attributes:nil] == NO)
     {
@@ -557,10 +708,13 @@
     }
 
   cvsWriter = [[DBCVSWriter alloc] initWithHandle:fileHandleOut];
-
+  [cvsWriter setLogger:logger];
+  progress = [[DBProgress alloc] init];
+  [progress setLogger:logger];
+  [progress reset];
 
   NS_DURING
-    [dbCsv queryIdentify :statement queryAll:([queryAllSelectIdentify state] == NSOnState) fromReader:cvsReader toWriter:cvsWriter withBatchSize:batchSize];
+    [dbCsv queryIdentify :statement queryAll:([queryAllSelectIdentify state] == NSOnState) fromReader:cvsReader toWriter:cvsWriter withBatchSize:batchSize progressMonitor:progress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -570,9 +724,10 @@
   NS_ENDHANDLER
 
   [cvsReader release];
-  
   [cvsWriter release];
   [fileHandleOut closeFile];
+  
+  [progress release];
 }
 
 /* DESCRIBE */
@@ -632,6 +787,7 @@
     }
   
   writer = [[DBCVSWriter alloc] initWithHandle:fileHandle];
+  [writer setLogger:logger];
   [writer setStringEncoding: [[defaults valueForKey: @"StringEncoding"] intValue]];
   
   whichObject = [[[popupObjectsDescribe selectedItem] title] retain];
@@ -743,9 +899,8 @@
   DBCVSReader   *reader;
   
   filePath = [fieldFileDelete stringValue];
-  NSLog(@"%@", filePath);
     
-  reader = [[DBCVSReader alloc] initWithPath:filePath byParsingHeaders:([checkSkipFirstLine state]==NSOnState)];  
+  reader = [[DBCVSReader alloc] initWithPath:filePath byParsingHeaders:([checkSkipFirstLine state]==NSOnState) withLogger:logger];  
   
   NS_DURING
     [dbCsv deleteFromReader:reader];
