@@ -1,7 +1,7 @@
 /*
  Project: DataBasin
  
- Copyright (C) 2010-2012 Free Software Foundation
+ Copyright (C) 2010-2013 Free Software Foundation
  
  Author: Riccardo Mottola
  
@@ -51,6 +51,26 @@
 
 - (void)awakeFromNib
 {
+  NSTableColumn *col;
+  NSCell *cell;
+
+  col = [fieldTable tableColumnWithIdentifier:COLID_LABEL];
+  cell = [col dataCell];
+  [cell setSelectable:YES];
+  [cell setEditable:NO];
+  [col setDataCell:cell];
+  
+  col = [fieldTable tableColumnWithIdentifier:COLID_DEVNAME];
+  cell = [col dataCell];
+  [cell setSelectable:YES];
+  [cell setEditable:NO];
+  [col setDataCell:cell];
+  
+  col = [fieldTable tableColumnWithIdentifier:COLID_VALUE];
+  cell = [col dataCell];
+  [cell setSelectable:YES];
+  [cell setEditable:NO];
+  [col setDataCell:cell];
 }
 
 - (void)show
@@ -130,12 +150,12 @@
 /** --- Data Source --- **/
 
 
-- (int)numberOfRowsInTableView:(NSTableView *)aTableView
+- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
 {
   return [arrayRows count];
 }
 
-- (id) tableView: (NSTableView*)aTableView objectValueForTableColumn: (NSTableColumn*)column row: (int)rowIndex
+- (id) tableView: (NSTableView*)aTableView objectValueForTableColumn: (NSTableColumn*)column row: (NSInteger)rowIndex
 {
   id retObj;
   NSDictionary *row;
