@@ -1,7 +1,7 @@
 /*
    Project: DataBasin
 
-   Copyright (C) 2008-2012 Free Software Foundation
+   Copyright (C) 2008-2013 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -23,19 +23,29 @@
 */
 
 #import <Foundation/Foundation.h>
-
 #import <WebServices/WebServices.h>
-#import "DBCVSWriter.h"
-#import "DBCVSReader.h"
+
+#import "DBLogger.h"
+
+#if defined(__APPLE__) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
+#ifndef NSUInteger
+#define NSUInteger unsigned int
+#endif
+#ifndef NSInteger
+#define NSInteger int
+#endif
+#endif
 
 #define MAX_SOQL_SIZE 9000
 #define MAX_BATCH_SIZE 200
 
 @class DBSObject;
+@protocol DBProgressProtocol;
 
 @interface DBSoap : NSObject
 {
   GWSService *service;
+  DBLogger *logger;
     
   /* salesforce.com session variables */
   NSString     *sessionId;
@@ -50,16 +60,24 @@
 
   /* create, update, upsert batch size */
   unsigned upBatchSize;
+
+  /** Timeout in seconds, for generic methods */
+  unsigned standardTimeoutSec;
+
+  /** Timeout in seconds, for query methods */
+  unsigned queryTimeoutSec;
 }
 
 - (void)login :(NSURL *)url :(NSString *)userName :(NSString *)password :(BOOL)useHttps;
+- (void)setLogger: (DBLogger *)l;
+- (DBLogger *)logger;
 
-- (NSMutableArray *)queryFull :(NSString *)queryString queryAll:(BOOL)all;
-- (NSString *)query :(NSString *)queryString queryAll:(BOOL)all toArray:(NSMutableArray *)objects;
+- (NSMutableArray *)queryFull :(NSString *)queryString queryAll:(BOOL)all progressMonitor:(id<DBProgressProtocol>)p;
+- (NSString *)query :(NSString *)queryString queryAll:(BOOL)all toArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p;
 - (NSString *)queryMore :(NSString *)locator toArray:(NSMutableArray *)objects;
-- (void)queryIdentify :(NSString *)queryString with: (NSString *)identifier queryAll:(BOOL)all fromArray:(NSArray *)fromArray toArray:(NSMutableArray *)outArray withBatchSize:(int)batchSize;
-- (void)create :(NSString *)objectName fromArray:(NSMutableArray *)objects;
-- (void)update :(NSString *)objectName fromArray:(NSMutableArray *)objects;
+- (void)queryIdentify :(NSString *)queryString with: (NSArray *)identifiers queryAll:(BOOL)all fromArray:(NSArray *)fromArray toArray:(NSMutableArray *)outArray withBatchSize:(int)batchSize progressMonitor:(id<DBProgressProtocol>)p;
+- (void)create :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p;
+- (void)update :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p;
 - (NSMutableArray *)delete :(NSArray *)objectIdArray;
 - (NSArray *)describeGlobal;
 - (NSArray *)sObjects;
@@ -73,6 +91,11 @@
 - (BOOL) passwordExpired;
 - (NSDictionary *) userInfo;
 
+- (void)setStandardTimeout:(unsigned)sec;
+- (void)setQueryTimeout:(unsigned)sec;
+- (unsigned)standardTimeout;
+- (unsigned)queryTimeout;
+
 @end
 
 
